package me.hammerle.snuviengine;

import java.util.ArrayList;
import java.util.Random;
import me.hammerle.snuviengine.api.Gamepad;
import me.hammerle.snuviengine.api.Renderer;
import me.hammerle.snuviengine.api.Game;
import me.hammerle.snuviengine.api.Keys;
import me.hammerle.snuviengine.api.Stats;

public class TestGame implements Game {
    private static final float MIN_X = 0.0f;
    private static final float MIN_Y = 0.0f;
    private static final float MAX_X = 200.0f;
    private static final float MAX_Y = 300.0f;

    private static final float SIZE = 12.0f;

    private Stats stats = null;
    private Gamepad gamepad = null;
    private Keys keys = null;

    private static class Entity {
        private float lastPosX;
        private float lastPosY;

        private float posX;
        private float posY;

        private float motionX;
        private float motionY;

        private int color = 0xFF0000FF;

        public boolean collidesWith(Entity e) {
            return posX < e.posX + SIZE && posX + SIZE > e.posX
                    && posY < e.posY + SIZE && posY + SIZE > e.posY;
        }

        public void collide(Entity e) {
            if(!collidesWith(e)) {
                return;
            }

            color = 0xFFFF00FF;
            e.color = 0xFFFF00FF;

            float diffX = SIZE - (e.posX - posX);
            float diffY = SIZE - (e.posY - posY);

            motionX += diffX;
            motionY = diffY;
            e.motionX -= diffX;
            e.motionY -= diffY;
        }
    }

    private final Random r = new Random(0);
    private final ArrayList<Entity> entities = new ArrayList<>();

    @Override
    public void onCompleteInitialization(Stats stats, Keys keys, Gamepad gamepad) {
        this.stats = stats;
        this.gamepad = gamepad;
        this.keys = keys;

        for(int i = 0; i < 2; i++) {
            Entity e = new Entity();
            e.posX = MIN_X + r.nextFloat() * (MAX_X - MIN_X - SIZE);
            e.posY = MIN_Y + r.nextFloat() * (MAX_Y - MIN_Y - SIZE);
            e.lastPosX = e.posX;
            e.lastPosY = e.posY;
            double angle = r.nextDouble() * 2.0 * Math.PI;
            double radius = 3.0;
            e.motionX = (float) (Math.sin(angle) * radius);
            e.motionY = (float) (Math.cos(angle) * radius);
            entities.add(e);
        }
        entities.get(0).posX = 100;
        entities.get(1).posX = 100;
        entities.get(0).lastPosX = 100;
        entities.get(1).lastPosX = 100;
        entities.get(0).motionX = 0;
        entities.get(1).motionX = 0;
    }

    @Override
    public void tick() {
        entities.forEach(e -> {
            e.motionY += 2.0f;

            e.lastPosX = e.posX;
            e.lastPosY = e.posY;
            e.posX += e.motionX;
            e.posY += e.motionY;

            if(e.posX < MIN_X) {
                e.posX = MIN_X;
                e.motionX = -e.motionX;
            }
            if(e.posY < MIN_Y) {
                //e.posY = MIN_Y;
                //e.motionY = -e.motionY;
            }
            if(e.posX + SIZE > MAX_X) {
                e.posX = MAX_X - SIZE;
                e.motionX = -e.motionX;
            }
            if(e.posY + SIZE > MAX_Y) {
                e.posY = MAX_Y - SIZE;
                e.motionY = -e.motionY;
            }
            
            e.color = 0xFF0000FF;
        });
        for(int i = 0; i < entities.size(); i++) {
            for(int k = i + 1; k < entities.size(); k++) {
                entities.get(i).collide(entities.get(k));
            }
        }
    }

    @Override
    public void renderTick(Renderer renderer, float lag) {        
        renderer.translateTo(0.0f, 0.0f);
        renderer.updateMatrix();

        renderer.setColorEnabled(true);
        renderer.setTextureEnabled(false);

        for(Entity e : entities) {
            float rx = e.lastPosX + (e.posX - e.lastPosX) * lag;
            float ry = e.lastPosY + (e.posY - e.lastPosY) * lag;
            renderer.getColorRenderer().drawRectangle(rx, ry, rx + SIZE, ry + SIZE, e.color);
        }

        renderer.setColorEnabled(true);
        renderer.setTextureEnabled(true);

        float y = renderer.getFontRenderer().drawString(10, 10, true, String.format("FPS: %.1f", stats.getFramesPerSecond()));
        renderer.getFontRenderer().drawString(10, y, true, String.format("TPS: %.1f", stats.getTicksPerSecond()));
    }

    @Override
    public void onStop() {
    }
}
